@@ -248,9 +248,19 @@ class VirtualField(FakeField): # (Field):
         
 class Constant(object):
     #~ get = None
-    def __init__(self,text):
-        self.text = text
+    def __init__(self,text_fn):
+        self.text_fn = text_fn
         
+def constant(verbose_name):
+    """
+    Decorator to turn a method into a Constant.
+    """
+    def decorator(fn):
+        def wrapped(*args):
+            #~ return classmethod(fn(*args))
+            return fn(*args)
+        return Constant(wrapped)
+    return decorator
 
 
 class RequestField(VirtualField):
@@ -520,25 +530,25 @@ class GenericForeignKey(generic.GenericForeignKey):
     Used by :class:`lino.mixins.Owned`.
     """
     def __init__(self, ct_field="content_type", fk_field="object_id", 
-          verbose_name=None):
+          verbose_name=None,help_text=None):
         self.verbose_name = verbose_name
+        self.help_text = help_text
         generic.GenericForeignKey.__init__(self,ct_field,fk_field)
         
     
-class FieldSet:
-    """
-    A group of fields that have a common label (`verbose_name`)
-    to be displayed and translated.
-    """
-    def __init__(self,verbose_name,desc=None,**child_labels):
-        self.verbose_name = verbose_name
-        self.desc = desc
-        self.child_labels = child_labels
+#~ class FieldSet:
+    #~ """
+    #~ A group of fields that have a common label (`verbose_name`)
+    #~ to be displayed and translated.
+    #~ """
+    #~ def __init__(self,verbose_name,desc=None,**child_labels):
+        #~ self.verbose_name = verbose_name
+        #~ self.desc = desc
+        #~ self.child_labels = child_labels
         
-    def get_child_label(self,name):
-        s = self.child_labels.get(name,None)
-        #~ logger.info('get_child_label(%r)->%s',name,unicode(s))
-        return s
+    #~ def get_child_label(self,name):
+        #~ s = self.child_labels.get(name,None)
+        #~ return s
         
 
 
