@@ -21,12 +21,14 @@ from django.db import models
 from django.contrib.contenttypes.models import ContentType
 from django.contrib.contenttypes import generic
 from django.utils.translation import ugettext as _
+from django.utils.translation import string_concat
 from django.conf import settings
 
 import lino
 
 from lino import dd
 from lino.core import table
+from lino.core import layouts
 from lino.core import fields
 from lino.utils import constrain
 from lino.utils import jsgen
@@ -42,6 +44,7 @@ EXT_CHAR_HEIGHT = 22
 #~ DEFAULT_GC_NAME = 'std'
 DEFAULT_GC_NAME = 0
 
+DEFAULT_PADDING = 2
 
 def form_field_name(f):
     if isinstance(f,models.ForeignKey) \
@@ -55,15 +58,13 @@ def form_field_name(f):
 def rpt2url(rpt):
     return '/' + rpt.app_label + '/' + rpt.__name__
 
-def a2btn(a):
-    return dict(
-      opens_a_slave=a.opens_a_slave,
-      handler=js_code("Lino.%s" % a),
-      name=a.name,
-      label=a.label, # 20111111
-      #~ label=unicode(a.label),
-      #~ url="/".join(("/ui",a.actor.app_label,a.actor._actor_name,a.name))
-    )
+#~ def a2btn(a):
+    #~ return dict(
+      #~ opens_a_slave=a.opens_a_slave,
+      #~ handler=js_code("Lino.%s" % a),
+      #~ name=a.name,
+      #~ label=a.label, # 20111111
+    #~ )
       
 def py2html(obj,name):
     for n in name.split('.'):
@@ -247,7 +248,7 @@ class VisibleComponent(Component):
         sep = u"</td><td>"
         cols = """ext_name name parent label __class__.__name__ 
         elements js_value
-        labelAlign vertical width preferred_width height 
+        label_align vertical width preferred_width height 
         preferred_height vflex""".split()
         yield '<tr><td>' + sep.join(cols) + '</td></tr>'
         for e in self.walk():
@@ -287,15 +288,12 @@ class LayoutElement(VisibleComponent):
         VisibleComponent.__init__(self,name,**kw)
         self.layout_handle = layout_handle
         #~ if layout_handle is not None:
-        assert isinstance(layout_handle,table.LayoutHandle)
+        assert isinstance(layout_handle,layouts.LayoutHandle)
         #~ layout_handle.setup_element(self)
 
     #~ def submit_fields(self):
         #~ return []
         
-    def update_config(self,wc):
-        pass
-        
     def get_property(self,name):
         v = getattr(self,name,None)
         if self.parent is None or v is not None:
@@ -312,16 +310,29 @@ class LayoutElement(VisibleComponent):
         #~ if isinstance(parent,FieldSetPanel):
             #~ self.label = None
             #~ self.update(label = None)
-        if self.label and isinstance(parent,Panel):
-            if parent.labelAlign == table.LABEL_ALIGN_LEFT:
-                self.preferred_width += len(self.label)
+        if self.label:
+            if isinstance(parent,Panel):
+                if parent.label_align == layouts.LABEL_ALIGN_LEFT:
+                    self.preferred_width += len(self.label)
 
     def ext_options(self,**kw):
-        kw = VisibleComponent.ext_options(self,**kw)
+        if isinstance(self.parent,TabPanel):
+            if not self.label:
+                raise Exception("Item %s of TabPanel %s has no label!" % (self,self.parent))
+            #~ if self.value_template != 'new Lino.VBorderPanel(%s)':
+            #~ if self.value_template == "%s":
+            #~ if self.value_template == "new Ext.Container":
+            #~ if self.value_template == "new Ext.Container(%s)":
+                #~ self.value_template = "new Ext.Panel(%s)"
+            #~ else:
+                #~ logger.info("20120217 value_template %s",self.value_template)
+            self.update(title=self.label)
+            self.update(listeners=dict(activate=js_code("Lino.on_tab_activate")))
         if self.xtype is not None:
-            kw.update(xtype=self.xtype)
+            self.update(xtype=self.xtype)
         if self.collapsible:
-            kw.update(collapsible=True)
+            self.update(collapsible=True)
+        kw = VisibleComponent.ext_options(self,**kw)
         return kw
 
 
@@ -334,7 +345,8 @@ class ConstantElement(LayoutElement):
     vflex = True
     
     def __init__(self,lh,fld,**kw):
-        kw.update(html=fld.text)
+        kw.update(html=fld.text_fn(lh.layout._table,lh.ui))
+        #~ kw.update(html=fld.text)
         #~ kw.update(autoHeight=True)
         LayoutElement.__init__(self,lh,fld.name,**kw)
         #~ self.text = text
@@ -373,18 +385,37 @@ class FieldElement(LayoutElement):
         self.editable = field.editable # and not field.primary_key
         
         #~ help_text = getattr(self.field,'help_text',None):
-        if self.field.help_text:
+        if settings.LINO.use_quicktips and self.field.help_text:
             #~ kw.update(qtip=self.field.help_text)
             #~ kw.update(toolTipText=self.field.help_text)
             #~ kw.update(tooltip=self.field.help_text)
-            kw.update(listeners=dict(render=js_code("Lino.quicktip_renderer(%s)" % py2js(self.field.help_text)))
-            )
+            kw.update(listeners=dict(render=js_code(
+              "Lino.quicktip_renderer(%s,%s)" % (py2js(self.field.verbose_name),py2js(self.field.help_text)))
+            ))
             
 
         #~ http://www.rowlands-bcs.com/extjs/tips/tooltips-form-fields
         #~ if self.field.__doc__:
             #~ kw.update(toolTipText=self.field.__doc__)
+        label = field.verbose_name
+        #~ if not self.field.blank:
+            #~ label = string_concat('<b>',label,'</b>')
+            #~ label = string_concat(label,' [*]')
+            
+        #~ kw.update(style=dict(padding=DEFAULT_PADDING))
+        #~ kw.update(style=dict(marginLeft=DEFAULT_PADDING))
+        #~ kw.update(style='padding: 10px')
+        
         kw.setdefault('label',field.verbose_name)
+        #~ kw.setdefault('label',string_concat('<b>',field.verbose_name,'</b>'))
+        #~ kw.setdefault('label',
+          #~ string_concat('<span class="ttdef"><a class="tooltip" href="#">',
+            #~ field.verbose_name,
+            #~ '<span class="classic">This is a test...</span></a></span>'))
+        #~ kw.setdefault('label',
+          #~ string_concat('<div class="ttdef"><a class="tooltip" href="#">',
+            #~ field.verbose_name,
+            #~ '<span class="classic">This is a test...</span></a></div>'))
         self.add_default_value(kw)
         #~ kw.update(label=field.verbose_name) 
         LayoutElement.__init__(self,layout_handle,field.name,**kw)
@@ -392,6 +423,16 @@ class FieldElement(LayoutElement):
     def cell_html(self,ui,row):
         return getattr(row,self.field.name)
             
+    #~ def submit_fields(self):
+        #~ return [self.field.name]
+    def add_default_value(self,kw):
+        if self.field.has_default():
+            dv = self.field.default
+            if callable(dv):
+                dv = dv()
+            kw.update(value=dv)
+            
+        
     def get_column_options(self,**kw):
         #~ raise "get_column_options() %s" % self.__class__
         #~ kw.update(xtype='gridcolumn')
@@ -410,16 +451,6 @@ class FieldElement(LayoutElement):
         return kw    
         
         
-    #~ def submit_fields(self):
-        #~ return [self.field.name]
-    def add_default_value(self,kw):
-        if self.field.has_default():
-            dv = self.field.default
-            if callable(dv):
-                dv = dv()
-            kw.update(value=dv)
-            
-        
     def get_field_options(self,**kw):
         if self.xtype:
             kw.update(xtype=self.xtype)
@@ -427,11 +458,27 @@ class FieldElement(LayoutElement):
         # When used as editor of an EditorGridPanel, don't set the name attribute
         # because it is not needed for grids and might conflict with fields of a 
         # surronding detail form. See ticket #38 (:doc:`/blog/2011/0408`).
-        if not isinstance(self.layout_handle.layout,table.ListLayout):
+        if not isinstance(self.layout_handle.layout,layouts.ListLayout):
             kw.update(name=self.field.name)
             if self.label:
+                label = self.label
+                if self.field.help_text and settings.LINO.use_css_tooltips:
+                    #~ label = string_concat(
+                      #~ '<b>',
+                      #~ field.verbose_name,'</b>')
+                    #~ label = string_concat(
+                      #~ '<span style:border-bottom: 1px dotted #000000; color: #000000; outline: none;>',
+                      #~ label,'</span>')
+                    label = string_concat(
+                      '<a class="tooltip" href="#">',
+                      label,
+                      '<span class="classic">',
+                      self.field.help_text,
+                      '</span></a>')
+                    #~ label = string_concat(label,' [?]')
+            
                 #~ kw.update(fieldLabel=unicode(self.label)) 20111111
-                kw.update(fieldLabel=self.label)
+                kw.update(fieldLabel=label)
         if self.editable:
             if not self.field.blank:
                 kw.update(allowBlank=False)
@@ -520,6 +567,8 @@ class CharFieldElement(FieldElement):
     def get_field_options(self,**kw):
         kw = FieldElement.get_field_options(self,**kw)
         kw.update(maxLength=self.field.max_length)
+        #~ kw.update(style=dict(padding=DEFAULT_PADDING))
+        #~ kw.update(margins='10px')
         return kw
         
         
@@ -558,7 +607,7 @@ class ComboFieldElement(FieldElement):
         # because it is not needed for grids and might conflict with fields of a 
         # surronding detail form. See ticket #38 (:doc:`/blog/2011/0408`).
         # Also, Comboboxes with simple values may never have a hiddenName option.
-        if not isinstance(self.layout_handle.layout,table.ListLayout) \
+        if not isinstance(self.layout_handle.layout,layouts.ListLayout) \
             and not isinstance(self,SimpleRemoteComboFieldElement):
             kw.update(hiddenName=self.field.name+ext_requests.CHOICES_HIDDEN_SUFFIX)
         return kw
@@ -586,8 +635,8 @@ class RemoteComboFieldElement(ComboFieldElement):
             url = self.layout_handle.ui.build_url("choices",
                 #~ self.layout_handle.layout.table.model._meta.app_label,
                 #~ self.layout_handle.layout.table.model.__name__,
-                self.layout_handle.layout.table.app_label,
-                self.layout_handle.layout.table.__name__,
+                self.layout_handle.layout._table.app_label,
+                self.layout_handle.layout._table.__name__,
                 self.field.name,**kw)
             proxy = dict(url=url,method='GET')
             kw.update(proxy=js_code("new Ext.data.HttpProxy(%s)" % py2js(proxy)))
@@ -634,7 +683,7 @@ class ForeignKeyElement(ComplexRemoteComboFieldElement):
         self.report = table.get_model_report(field.rel.to)
         a = self.report.detail_action
         if a is not None:
-            if not isinstance(layout_handle.layout,table.ListLayout):
+            if not isinstance(layout_handle.layout,layouts.ListLayout):
                 self.value_template = "new Lino.TwinCombo(%s)"
                 kw.update(onTrigger2Click=js_code(
                     "function(){ Lino.show_fk_detail(this,Lino.%s)}" % a))
@@ -828,7 +877,7 @@ class BooleanFieldElement(FieldElement):
 
     def get_field_options(self,**kw):
         kw = FieldElement.get_field_options(self,**kw)
-        if not isinstance(self.layout_handle.layout,table.ListLayout):
+        if not isinstance(self.layout_handle.layout,layouts.ListLayout):
             if kw.has_key('fieldLabel'):
                 del kw['fieldLabel']
             #~ kw.update(hideLabel=True)
@@ -932,7 +981,8 @@ class Container(LayoutElement):
     is_fieldset = False
     #~ xtype = 'container'
     value_template = "new Ext.Container(%s)"
-    #~ hideCheckBoxLabels = True
+    hideCheckBoxLabels = True
+    label_align = layouts.LABEL_ALIGN_TOP
     
     #declare_type = jsgen.DECLARE_INLINE
     declare_type = jsgen.DECLARE_VAR
@@ -941,14 +991,18 @@ class Container(LayoutElement):
     
     
     def __init__(self,layout_handle,name,*elements,**kw):
-        self.has_frame = layout_handle.layout.has_frame
-        self.labelAlign = layout_handle.layout.label_align
-        self.hideCheckBoxLabels = layout_handle.layout.hideCheckBoxLabels
+        #~ self.has_frame = layout_handle.layout.has_frame
+        #~ self.labelAlign = layout_handle.layout.label_align
+        #~ self.hideCheckBoxLabels = layout_handle.layout.hideCheckBoxLabels
         self.active_children = []
         self.elements = elements
         if elements:
             #~ self.has_fields = False
             for e in elements:
+                #~ v = getattr(self,e.name,None)
+                #~ if v is not None:
+                    #~ raise Exception("%s has already %s = %s" % (self,e.name,v))
+                #~ setattr(self,e.name,e)
                 e.set_parent(self)
                 #~ if isinstance(e,FieldElement):
                     #~ self.has_fields = True
@@ -998,7 +1052,8 @@ class Wrapper(VisibleComponent):
         kw.update(layout='form')
         if not isinstance(e,TextFieldElement):
             kw.update(autoHeight=True)
-        kw.update(labelAlign=e.parent.labelAlign)
+        #~ kw.update(labelAlign=e.parent.labelAlign)
+        kw.update(labelAlign=e.parent.label_align)
         kw.update(items=e,xtype='panel')
         VisibleComponent.__init__(self,e.name+"_ct",**kw)
         self.wrapped = e
@@ -1009,6 +1064,7 @@ class Wrapper(VisibleComponent):
             e.update(anchor="100% 100%")
         else:
             e.update(anchor="100%")
+        #~ e.update(padding=DEFAULT_PADDING)
             
     def walk(self):
         for e in self.wrapped.walk():
@@ -1024,6 +1080,7 @@ class Panel(Container):
     """
     ext_suffix = "_panel"
     active_child = False
+    value_template = "new Ext.Panel(%s)"
     #~ declare_type = jsgen.DECLARE_VAR
     #~ value_template = "new Ext.Panel(%s)"
     
@@ -1122,11 +1179,12 @@ class Panel(Container):
                         self.label_width = w
 
 
-        label = layout_handle.layout.collapsible_elements.get(name,None)
-        if label:
-            self.collapsible = True
-            self.label = label
-
+        if False: # not yet converted to new dtl syntax 20120214
+            label = layout_handle.layout.collapsible_elements.get(name,None)
+            if label:
+                self.collapsible = True
+                self.label = label
+            
         Container.__init__(self,layout_handle,name,*elements,**kw)
 
         w = h = 0
@@ -1171,7 +1229,8 @@ class Panel(Container):
                 
         if d['layout'] == 'form':
             assert self.vertical
-            self.update(labelAlign=self.labelAlign)
+            #~ self.update(labelAlign=self.labelAlign)
+            self.update(labelAlign=self.label_align)
             self.wrap_formlayout_elements()
             #~ d.update(autoHeight=True)
             if len(self.elements) == 1 and self.elements[0].vflex:
@@ -1236,6 +1295,23 @@ class Panel(Container):
             self.wrap_formlayout_elements()
         else:
             raise Exception("layout is %r" % d['layout'] )
+            
+        if self.is_fieldset:
+            self.update(labelWidth=self.label_width * EXT_CHAR_WIDTH)
+        #~ else:
+            #~ d.update(padding=DEFAULT_PADDING)
+        if len(self.elements) > 1 and self.vertical:
+            #d.update(frame=self.has_frame)
+            self.update(frame=True)
+            self.update(bodyBorder=False)
+            self.update(border=False)
+            #~ 20120115 d.update(labelAlign=self.labelAlign)
+            #d.update(style=dict(padding='0px'),color='green')
+        else:
+            self.update(frame=False)
+            #self.update(bodyBorder=False)
+            self.update(border=False)
+            
         
     def wrap_formlayout_elements(self):
         #~ if layout_handle.main_class is DetailMainPanel:
@@ -1253,6 +1329,25 @@ class Panel(Container):
           
         
     def ext_options(self,**d):
+        #~ if not self.label and self.value_template == "new Ext.Panel(%s)":
+            # if not self.parent or len(self.parent.elements) == 1:
+            # if self.parent and len(self.parent.elements) > 1:
+            #~ if self.parent is not None:
+                #~ self.value_template = "new Ext.Container(%s)"
+            
+        if self.label:
+            if not isinstance(self.parent,TabPanel):
+                self.update(title=self.label)
+                self.value_template = "new Ext.form.FieldSet(%s)"
+                self.update(frame=False)
+                self.update(bodyBorder=True)
+                self.update(border=True)
+            #~ elif isinstance(self.parent,DetailMainPanel):
+                #~ if self.value_template == "new Ext.Container(%s)":
+                    #~ self.value_template = "new Ext.Panel(%s)"
+              
+            #~ else:
+                #~ self.value_template = "new Ext.Panel(%s)"
         d = Container.ext_options(self,**d)
         #~ if self.collapsible:
             #~ d.update(xtype='panel')
@@ -1268,48 +1363,9 @@ class Panel(Container):
         #l = [e.as_ext() for e in self.elements ]
         #d.update(items=js_code("[\n  %s\n]" % (", ".join(l))))
         #d.update(items=js_code("this.elements"))
-        
-        if self.is_fieldset:
-            d.update(labelWidth=self.label_width * EXT_CHAR_WIDTH)
-        if len(self.elements) > 1 and self.vertical:
-            #d.update(frame=self.has_frame)
-            d.update(frame=True)
-            d.update(bodyBorder=False)
-            d.update(border=False)
-            #~ 20120115 d.update(labelAlign=self.labelAlign)
-            #d.update(style=dict(padding='0px'),color='green')
-        else:
-            d.update(frame=False)
-            #d.update(bodyBorder=False)
-            d.update(border=False)
-            
-        if self.label:
-            #~ d.update(title=unicode(self.label)) 20111111
-            d.update(title=self.label)
-            
+        #~ if self.label and not isinstance(self,MainPanel) and not isinstance(self.parent,MainPanel):
         return d
         
-class FieldSetPanel(Panel):
-    value_template = "new Ext.form.FieldSet(%s)"
-    def __init__(self,layout_handle,name,vertical,*elements,**kw):
-        self.fieldset = getattr(layout_handle.layout.table.model,name)
-        for child in elements:
-            child.label = self.fieldset.get_child_label(child.name)
-        Panel.__init__(self,layout_handle,name,vertical,*elements,**kw)
-        
-        self.label = self.fieldset.verbose_name
-        
-    def ext_options(self,**d):
-        d = Panel.ext_options(self,**d)
-        d.update(frame=False)
-        d.update(bodyBorder=True)
-        d.update(border=True)
-        d.update(labelAlign=self.labelAlign)
-        return d
-        
-    #~ def wrap_formlayout_elements(self):
-        #~ pass
-  
 
 class GridElement(Container): 
     declare_type = jsgen.DECLARE_VAR
@@ -1335,7 +1391,7 @@ class GridElement(Container):
             self.rh = rpt.get_handle(layout_handle.ui)
             if not hasattr(self.rh,'list_layout'):
                 raise Exception("%s has no list_layout" % self.rh)
-            columns = self.rh.list_layout._main.columns
+            columns = self.rh.list_layout.main.columns
             #~ columns = self.rh.list_layout._main.elements
         w = 0
         for e in columns:
@@ -1357,7 +1413,7 @@ class GridElement(Container):
         #~ //~ enableRowBody: true,
         
         kw.update(viewConfig=vc)
-        
+        kw.setdefault('label',rpt.label)
         
         #~ kw.update(containing_window=js_code("this.containing_window"))
         kw.update(containing_panel=js_code("this"))
@@ -1376,55 +1432,36 @@ class GridElement(Container):
             #~ self.mt = 'undefined'
             
             
-    def update_config(self,wc):
-        pass
-        #~ for i,w in enumerate(wc['column_widths']):
-            #~ self.column_model.columns[i].update(width = w)
-
     def ext_options(self,**kw):
         "not direct parent (Container), only LayoutElement"
         kw = LayoutElement.ext_options(self,**kw)
         return kw
         
-    def unused_ext_options(self,**kw):
-        rh = self.report.get_handle(self.layout_handle.ui)
-        kw = LayoutElement.ext_options(self,**kw)
-        kw.update(ls_url=rpt2url(self.report))
-        kw.update(ls_store_fields=[js_code(f.as_js()) for f in rh.store.list_fields])
-        kw.update(ls_columns=[GridColumn(i,e) for i,e in enumerate(self.columns)])
-        kw.update(ls_id_property=rh.store.pk.name)
-        kw.update(pk_index=rh.store.pk_index)
-        kw.update(ls_quick_edit=rh.report.cell_edit)
-        kw.update(ls_bbar_actions=[rh.ui.a2btn(a) for a in rh.get_actions(rh.report.default_action)])
-        kw.update(ls_grid_configs=[gc.data for gc in self.report.grid_configs])
-        kw.update(gc_name=DEFAULT_GC_NAME)
-        return kw
-        
         
 
             
-class MainPanel(jsgen.Variable):
-    declare_type = jsgen.DECLARE_INLINE
-    refers_to_ww = True
+#~ class MainPanel(jsgen.Variable):
+    #~ declare_type = jsgen.DECLARE_INLINE
+    #~ refers_to_ww = True
   
-    def apply_window_config(self,wc):
-        pass
-  
-    #~ def setup(self):
+    #~ def apply_window_config(self,wc):
         #~ pass
-                
+  
 
 
 
-class GridMainPanel(GridElement,MainPanel):
-    def __init__(self,layout_handle,name,vertical,*columns,**kw):
-        """ignore the "vertical" arg"""
-        GridElement.__init__(self,layout_handle,name,layout_handle.rh.report,*columns,**kw)
+#~ class GridMainPanel(GridElement,MainPanel):
+#~ class GridMainPanel(GridElement):
+    #~ pass
+    #~ def __init__(self,lh,name,vertical,*columns,**kw):
+        #~ """Note that this ignores the "vertical" arg"""
+        #~ GridElement.__init__(self,lh,name,lh.layout.table,*columns,**kw)
         
 
 
 
-class DetailMainPanel(Panel,MainPanel):
+#~ class DetailMainPanel(Panel,MainPanel):
+class DetailMainPanel(Panel):
     #~ declare_type = jsgen.DECLARE_THIS
     #~ xtype = 'form'
     xtype = None
@@ -1440,7 +1477,7 @@ class DetailMainPanel(Panel,MainPanel):
         Panel.__init__(self,layout_handle,name,vertical,*elements,**kw)
         #layout_handle.needs_store(self.rh)
         
-    def subvars(self):
+    def unused_subvars(self):
         #~ print 'DetailMainPanel.subvars()', self
         for e in MainPanel.subvars(self):
             yield e
@@ -1450,13 +1487,16 @@ class DetailMainPanel(Panel,MainPanel):
     def ext_options(self,**kw):
         #~ self.setup()
         kw = Panel.ext_options(self,**kw)
-        if self.layout_handle.layout.label:
-            kw.update(title=_(self.layout_handle.layout.label))
+        #~ if self.layout_handle.layout.label:
+            #~ kw.update(title=_(self.layout_handle.layout.label))
+        if self.layout_handle.main.label:
+            kw.update(title=_(self.layout_handle.main.label))
         return kw
         
 
 class ParameterPanel(DetailMainPanel):
-    value_template = "new Ext.form.FormPanel(%s)"
+    #~ value_template = "new Ext.form.FormPanel(%s)"
+    value_template = "new Ext.form.FormPanel({layout:'fit', autoHeight: true, frame: true, items:new Ext.Panel(%s)})"
     #~ pass
     
 class unused_ParameterPanel(DetailMainPanel):
@@ -1477,19 +1517,24 @@ class unused_ParameterPanel(DetailMainPanel):
         return kw
     
 
-class TabPanel(jsgen.Component):
+class TabPanel(Panel):
+#~ class TabPanel(Panel,MainPanel):
+#~ class TabPanel(jsgen.Component):
 #~ class TabPanel(jsgen.Value):
     value_template = "new Ext.TabPanel(%s)"
+    #~ width = None
+    #~ height = None
 
-    def __init__(self,tabs,**kw):
-        self.active_children = []
-        for t in tabs:
-            self.active_children += t.active_children
-            t.update(listeners=dict(activate=js_code("Lino.on_tab_activate")))
-            #~ if t.has_upload:
-                #~ self.has_upload = True
-      
-        self.tabs = tabs
+    def __init__(self,layout_handle,name,*elems,**kw):
+        #~ self.rh = layout_handle.datalink
+        #~ 20111126 self.report = layout_handle.rh.report
+        #~ MainPanel.__init__(self)
+        #~ DataElementMixin.__init__(self,layout_handle.link)
+        kw.update(autoScroll=True)
+        #~ kw.update(height=800, autoScroll=True)
+        
+        #~ self.tabs = elems
+        
         kw.update(
           split=True,
           activeTab=0,
@@ -1498,27 +1543,28 @@ class TabPanel(jsgen.Component):
           #~ deferredRender=False, # 20120212
           #~ autoScroll=True, 
           #~ width=300, # ! http://code.google.com/p/lino/wiki/20100513
-          items=tabs,
+          #~ items=elems,
           # http://www.extjs.com/forum/showthread.php?26564-Solved-FormPanel-in-a-TabPanel
           #~ listeners=dict(activate=js_code("function(p) {p.doLayout();}"),single=True),
         )
-        jsgen.Value.__init__(self,kw)
+        Container.__init__(self,layout_handle,name,*elems,**kw)
+        
+    #~ def __init__(self,tabs,**kw):
+        #~ self.active_children = []
+        #~ for t in tabs:
+            #~ self.active_children += t.active_children
+            #~ t.update(listeners=dict(activate=js_code("Lino.on_tab_activate")))
+      
+        #~ super(TabPanel,self).__init__(kw)
         
-    def unused_has_field(self,f):
-        for t in self.tabs:
-            if t.has_field(f): 
-                return True
 
 
-from lino.tools import full_model_name
-
 class FormPanel(jsgen.Component):
     declare_type = jsgen.DECLARE_VAR
     #~ listeners = None
     
     def __init__(self,rh,action,**kw):
         self.rh = rh
-        #~ self.value_template = "new Lino.%s.FormPanel(%%s)" % full_model_name(self.rh.report.model)
         self.value_template = "new Lino.%sPanel(%%s)" % action
         #~ hmm....
         #~ kw.update(
